# Copyright (c) 2015, Alejandro Guirao <lekumberri@gmail.com>
# Copyright (c) 2012-17 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: shelvefile
author: Alejandro Guirao (!UNKNOWN) <lekumberri@gmail.com>
short_description: Read keys from Python shelve file
description:
  - Read keys from Python shelve file.
options:
  _terms:
    description: Sets of key value pairs of parameters.
    type: list
    elements: str
  key:
    description: Key to query.
    type: str
    required: true
  file:
    description: Path to shelve file.
    type: path
    required: true
"""

EXAMPLES = r"""
---
- name: Retrieve a string value corresponding to a key inside a Python shelve file
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.shelvefile', 'file=path_to_some_shelve_file.db key=key_to_retrieve') }}"
"""

RETURN = r"""
_list:
  description: Value(s) of key(s) in shelve file(s).
  type: list
  elements: str
"""
import shelve

from ansible.errors import AnsibleAssertionError, AnsibleError
from ansible.module_utils.common.text.converters import to_bytes, to_text
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms


class LookupModule(LookupBase):
    def read_shelve(self, shelve_filename, key):
        """
        Read the value of "key" from a shelve file
        """
        d = shelve.open(to_bytes(shelve_filename))
        res = d.get(key, None)
        d.close()
        return res

    def run(self, terms, variables=None, **kwargs):
        # TODO: use new-style option parsing
        if not isinstance(terms, list):
            terms = [terms]

        check_for_wrong_terms(self, direct=kwargs)

        ret = []

        for term in terms:
            paramvals = {"file": None, "key": None}
            params = term.split()

            try:
                for param in params:
                    name, value = param.split("=")
                    if name not in paramvals:
                        raise AnsibleAssertionError(f"{name} not in paramvals")
                    paramvals[name] = value

            except (ValueError, AssertionError) as e:
                # In case "file" or "key" are not present
                raise AnsibleError(e) from e

            key = paramvals["key"]

            # Search also in the role/files directory and in the playbook directory
            shelvefile = self.find_file_in_search_path(variables, "files", paramvals["file"])

            if shelvefile:
                res = self.read_shelve(shelvefile, key)
                if res is None:
                    raise AnsibleError(f"Key {key} not found in shelve file {shelvefile}")
                # Convert the value read to string
                ret.append(to_text(res))
                break
            else:
                raise AnsibleError(f"Could not locate shelve file in lookup: {paramvals['file']}")

        return ret



#!/usr/bin/python
#
# Copyright (c) 2020 Haiyuan Zhang, <haiyzhan@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_adserviceprincipal

version_added: "0.2.0"

short_description: Manage Azure Active Directory service principal

description:
        - Manage Azure Active Directory service principal.

options:
    account_enabled:
        description:
            - Whether the service principal is enabled.
        type: bool
        default: True
    app_id:
        description:
            - The application ID.
        type: str
        required: True
    app_role_assignment_required:
        description:
            - Whether the Role of the Service Principal is set.
        type: bool
    notes:
        description:
            - Use this field to add information that is relevant for the management of the application
        type: str
    service_principal_type:
        description:
            - The type of the service principal.
        choices:
            - application
            - managedIdentity
        type: str
        default: application
    state:
        description:
            - Assert the state of Active Dirctory service principal.
            - Use C(present) to create or update a Password and use C(absent) to delete.
        default: present
        choices:
            - absent
            - present
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    haiyuan_zhang (@haiyuazhang)
    Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: create ad sp
  azure_rm_adserviceprincipal:
    app_id: "{{ app_id }}"
    state: present
'''

RETURN = '''
app_display_name:
    description:
        - Object's display name or its prefix.
    type: str
    returned: always
    sample: fredAKSCluster
app_id:
    description:
        - The application ID.
    returned: always
    type: str
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
app_role_assignment_required:
    description:
        - Whether the Role of the Service Principal is set.
    returned: always
    type: bool
    sample: false
object_id:
    description:
        - Object ID of the associated service principal.
    returned: always
    type: str
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from msgraph.generated.models.service_principal import ServicePrincipal
except Exception:
    pass

try:
    import asyncio
    from msgraph.generated.service_principals.service_principals_request_builder import ServicePrincipalsRequestBuilder
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADServicePrincipal(AzureRMModuleBaseExt):
    def __init__(self):

        self.module_arg_spec = dict(
            account_enabled=dict(type='bool', default=True),
            app_id=dict(type='str', required=True),
            notes=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            app_role_assignment_required=dict(type='bool'),
            service_principal_type=dict(type='str', default='application', choices=['application', 'managedIdentity'])
        )

        self.account_enabled = None
        self.state = None
        self.app_id = None
        self.app_role_assignment_required = None
        self.notes = None
        self.object_id = None
        self.service_principal_type = None
        self.results = dict(changed=False)

        super(AzureRMADServicePrincipal, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                        supports_check_mode=False,
                                                        supports_tags=False,
                                                        is_ad_resource=True)

        mutually_exclusive = {}

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self._client = self.get_msgraph_client()

        response = self.get_resource()

        if response:
            if self.state == 'present':
                if self.check_update(response):
                    self.update_resource(response)
            elif self.state == 'absent':
                self.delete_resource(response)
        else:
            if self.state == 'present':
                self.create_resource()
            elif self.state == 'absent':
                self.log("try to delete non exist resource")

        return self.results

    def create_resource(self):
        try:
            response = asyncio.get_event_loop().run_until_complete(self.create_service_principal())
            self.results['changed'] = True
            self.results.update(self.to_dict(response))
            return response
        except Exception as ge:
            self.fail("Error creating service principle, app id {0} - {1}".format(self.app_id, str(ge)))

    def update_resource(self, old_response):
        try:
            request_body = ServicePrincipal(
                app_role_assignment_required=None,
            )
            if self.app_role_assignment_required is not None:
                request_body = ServicePrincipal(
                    app_role_assignment_required=self.app_role_assignment_required
                )

            asyncio.get_event_loop().run_until_complete(self.update_service_principal(old_response, request_body))
            self.results['changed'] = True
            self.results.update(self.get_resource())

        except Exception as ge:
            self.fail("Error updating the service principal app_id {0} - {1}".format(self.app_id, str(ge)))

    def delete_resource(self, response):
        try:
            asyncio.get_event_loop().run_until_complete(self.delete_service_principal(response))
            self.results['changed'] = True
            return True
        except Exception as ge:
            self.fail("Error deleting service principal app_id {0} - {1}".format(self.app_id, str(ge)))

    def get_resource(self):
        try:
            sps = asyncio.get_event_loop().run_until_complete(self.get_service_principals())
            result = list(sps.value)
            if not result:
                return False
            result = result[0]
            return self.to_dict(result)
        except Exception as ge:
            self.log("Did not find the graph instance instance {0} - {1}".format(self.app_id, str(ge)))
            return False

    def check_update(self, response):
        app_assignment_changed = self.app_role_assignment_required is not None and \
            self.app_role_assignment_required != response.get('app_role_assignment_required', None)

        return app_assignment_changed

    def to_dict(self, object):
        return dict(
            account_enabled=object.account_enabled,
            app_id=object.app_id,
            notes=object.notes,
            object_id=object.id,
            app_display_name=object.display_name,
            app_role_assignment_required=object.app_role_assignment_required,
            service_principal_type=object.service_principal_type
        )

    async def create_service_principal(self):
        request_body = ServicePrincipal(
            app_id=self.app_id,
            account_enabled=self.account_enabled,
            notes=self.notes,
            service_principal_type=self.service_principal_type
        )
        return await self._client.service_principals.post(body=request_body)

    async def update_service_principal(self, old_response, request_body):
        return await self._client.service_principals.by_service_principal_id(old_response['object_id']).patch(
            body=request_body)

    async def delete_service_principal(self, response):
        return await self._client.service_principals.by_service_principal_id(response.get('object_id')).delete()

    async def get_service_principals(self):
        request_configuration = ServicePrincipalsRequestBuilder.ServicePrincipalsRequestBuilderGetRequestConfiguration(
            query_parameters=ServicePrincipalsRequestBuilder.ServicePrincipalsRequestBuilderGetQueryParameters(
                filter="servicePrincipalNames/any(c:c eq '{0}')".format(self.app_id),
            )
        )
        return await self._client.service_principals.get(request_configuration=request_configuration)


def main():
    AzureRMADServicePrincipal()


if __name__ == '__main__':
    main()



# Copyright (c) 2019-2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible.errors import AnsibleConnectionFailure
from ansible.utils.display import Display

from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClientBase,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DOCKER_COMMON_ARGS,
)

if t.TYPE_CHECKING:
    from ansible.plugins import AnsiblePlugin


class AnsibleDockerClient(AnsibleDockerClientBase):
    def __init__(
        self, plugin: AnsiblePlugin, min_docker_api_version: str | None = None
    ) -> None:
        self.plugin = plugin
        self.display = Display()
        super().__init__(min_docker_api_version=min_docker_api_version)

    def fail(self, msg: str, **kwargs: t.Any) -> t.NoReturn:
        if kwargs:
            msg += "\nContext:\n" + "\n".join(
                f"  {k} = {v!r}" for (k, v) in kwargs.items()
            )
        raise AnsibleConnectionFailure(msg)

    def deprecate(
        self,
        msg: str,
        version: str | None = None,
        date: str | None = None,
        collection_name: str | None = None,
    ) -> None:
        self.display.deprecated(
            msg, version=version, date=date, collection_name=collection_name
        )

    def _get_params(self) -> dict[str, t.Any]:
        return {option: self.plugin.get_option(option) for option in DOCKER_COMMON_ARGS}



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.routesoperations?view=azure-python
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_afdroute_info
version_added: "3.4.0"
short_description: Get Azure Front Door Route facts to be used with Standard or Premium Frontdoor Service
description:
    - Get facts for a specific Azure Front Door (AFD) Route or all AFD Routes.
    - This differs from the Front Door classic service and only is intended to be used by the Standard or Premium service offering.

options:
    endpoint_name:
        description:
            - Name of the endpoint under the profile which is unique globally.
        required: true
        type: str
    name:
        description:
            - Name of the route.
        type: str
    profile_name:
        description:
            - Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group
        required: true
        type: str
    resource_group:
        description:
            - Name of the Resource group within the Azure subscription.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Get facts for all Routes in the AFD Profile
  azure_rm_afdroute_info:
    endpoint_name: myEndpoint
    profile_name: myProfile
    resource_group: myResourceGroup

- name: Get facts of specific AFD Route
  azure_rm_afdroute_info:
    name: myRoute1
    endpoint_name: myEndpoint
    profile_name: myProfile
    resource_group: myResourceGroup
'''

RETURN = '''
afdroutes:
    description: List of AFD Routes.
    returned: always
    type: complex
    contains:
        content_types_to_compress:
            description:
                - List of content types (str) on which compression applies. The value should be a valid MIME type.
            type: list
        custom_domains:
            description:
                - Domain id's referenced by this endpoint.
            type: list
        deployment_status:
            description:
                - Current state of the resource.
            type: str
            sample: NotStarted
        enabled_state:
            description:
                - Whether to enable use of this rule.
            type: str
        endpoint_name:
            description:
                - Name of the endpoint.
            type: str
        forwarding_protocol:
            description:
                - Protocol this rule will use when forwarding traffic to backends.
            type: str
        https_redirect:
            description:
                - Whether to automatically redirect HTTP traffic to HTTPS traffic.
                - Note that this is a easy way to set up this rule and it will be the first rule that gets executed.
            type: str
        id:
            description:
                - ID of the AFD Route.
            type: str
            sample: "/subscriptions/xxxxx-xxxx-xxxx-xxxx-xxxxxx/resourcegroups/myRG/providers/Microsoft.Cdn/profiles/myProf/routegroups/myRtGrp/routes/myRt"
        is_compression_enabled:
            description:
                - Indicates whether content compression is enabled on AzureFrontDoor. If compression is enabled,
                - content will be served as compressed if user requests for a compressed version. Content won't be compressed
                - on AzureFrontDoor when requested content is smaller than 1 byte or larger than 1 MB.
            type: bool
        link_to_default_domain:
            description:
                - Whether this route will be linked to the default endpoint domain.
            type: str
        name:
            description:
                - Name of the AFD Route.
            type: str
        origin_group_id:
            description:
                - The origin group id.
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxx/resourcegroups/myRG/providers/Microsoft.Cdn/profiles/myProf/origingroups/myOG
        origin_path:
            description:
                - A directory path on the origin that AzureFrontDoor can use to retrieve content from, e.g. contoso.cloudapp.net/originpath.
            type: str
        patterns_to_match:
            description:
                - The route patterns of the rule.
            type: list
        profile_name:
            description:
                - Name of the AFD Profile where the Route is.
            type: str
        provisioning_state:
            description:
                - Provisioning status of the AFD Route.
            type: str
            sample: Succeeded
        query_parameters:
            description:
                - query parameters to include or exclude (comma separated).
            type: str
        query_string_caching_behavior:
            description:
                - Defines how Frontdoor caches requests that include query strings. You can ignore any query strings when caching,
                - ignore specific query strings, cache every request with a unique URL, or cache specific query strings.
            type: str
        resource_group_name:
            description:
                - Name of a resource group where the AFD Route exists.
            type: str
        rule_sets:
            description:
                - List of rule set id referenced by this endpoint.
            type: list
        supported_protocols:
            description:
                - List of supported protocols for this route.
            type: list
        type:
            description:
                - Resource type.
            type: str
            sample: Microsoft.Cdn/profiles/afdendpoints/routes
'''

import re
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'AFDRoute'


class AzureRMAFDRouteInfo(AzureRMModuleBase):
    """Utility class to get Azure AFD Route facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            endpoint_name=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            resource_group=dict(
                type='str',
                required=True
            )
        )

        self.results = dict(
            changed=False,
            afdroutes=[]
        )

        self.name = None
        self.endpoint_name = None
        self.resource_group = None
        self.profile_name = None

        super(AzureRMAFDRouteInfo, self).__init__(
            supports_check_mode=True,
            derived_arg_spec=self.module_args,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['afdroutes'] = self.get_item()
        else:
            self.results['afdroutes'] = self.list_by_endpoint()

        return self.results

    def get_item(self):
        """Get a single Azure AFD Route"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.cdn_client.routes.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                endpoint_name=self.endpoint_name,
                route_name=self.name
            )
        except Exception as exc:
            self.log("Failed to find an existing resource. {0}".format(str(exc)))

        if item:
            result = [self.serialize_afdroute(item)]

        return result

    def list_by_endpoint(self):
        """Get all Azure AFD Routes within an AFD profile"""

        self.log('List all AFD Routes within an AFD profile')

        try:
            response = self.cdn_client.routes.list_by_endpoint(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                endpoint_name=self.endpoint_name
            )
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            results.append(self.serialize_afdroute(item))

        return results

    def serialize_afdroute(self, afdroute):
        '''
        Convert a AFD Route object to dict.
        :param afdroute: AFD Route object
        :return: dict
        '''
        result = self.serialize_obj(afdroute, AZURE_OBJECT_CLASS)

        new_result = {}
        new_result['content_types_to_compress'] = None
        new_result['is_compression_enabled'] = None
        new_result['query_parameters'] = None
        new_result['query_string_caching_behavior'] = None
        if afdroute.cache_configuration:
            new_result['content_types_to_compress'] = \
                afdroute.cache_configuration.compression_settings.content_types_to_compress
            new_result['is_compression_enabled'] = \
                afdroute.cache_configuration.compression_settings.is_compression_enabled
            new_result['query_parameters'] = afdroute.cache_configuration.query_parameters
            new_result['query_string_caching_behavior'] = afdroute.cache_configuration.query_string_caching_behavior
        new_result['custom_domains'] = []
        for domain in afdroute.custom_domains:
            dom = {
                "id": domain.id,
                "is_active": domain.is_active
            }
            new_result['custom_domains'].append(dom)
        new_result['deployment_status'] = afdroute.deployment_status
        new_result['enabled_state'] = afdroute.enabled_state
        new_result['endpoint_name'] = self.endpoint_name
        new_result['forwarding_protocol'] = afdroute.forwarding_protocol
        new_result['https_redirect'] = afdroute.https_redirect
        new_result['id'] = afdroute.id
        new_result['link_to_default_domain'] = afdroute.link_to_default_domain
        new_result['name'] = afdroute.name
        new_result['origin_group_id'] = afdroute.origin_group.id
        new_result['origin_path'] = afdroute.origin_path
        new_result['patterns_to_match'] = afdroute.patterns_to_match
        new_result['provisioning_state'] = afdroute.provisioning_state
        new_result['rule_sets'] = []
        for rule_set in afdroute.rule_sets:
            new_result['rule_sets'].append(rule_set.id)
        new_result['profile_name'] = re.sub('\\/.*', '', re.sub('.*profiles\\/', '', result['id']))
        new_result['resource_group_name'] = re.sub('\\/.*', '', re.sub('.*resourcegroups\\/', '', result['id']))
        new_result['supported_protocols'] = afdroute.supported_protocols
        new_result['type'] = afdroute.type
        return new_result


def main():
    """Main module execution code path"""
    AzureRMAFDRouteInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_tags
version_added: "3.5.0"
short_description: Manage tags
description:
    - Create, update ,delete and replace the tags.
options:
    scope:
        description:
            - The resource scope.
        type: str
        required: True
    operation:
        description:
            - The operation type for the patch API.
            - The default value is I(operation=Merge) and use to add tags.
        type: str
        default: Merge
        choices:
            - Delete
            - Replace
            - Merge
    state:
        description:
            - State of the SSH Public Key. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - magodo (@magodo)
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create a new tags with scope
  azure_rm_tags:
    scope: "/subscriptions/xxxxxxxxxxxxxxxxxxxxxxxxxx/resourceGroups/v-xisuRG02"
    tags:
      key1: value1

- name: Update the tags with scope
  azure_rm_tags:
    scope: "/subscriptions/xxxxxxxxxxxxxxxxxxxxxxxxxx/resourceGroups/v-xisuRG02"
    operation: Merge
    tags:
      key2: value2

- name: Delete the tags by scope
  azure_rm_tags:
    scope: "/subscriptions/xxxxxxxxxxxxxxxxxxxxxxxxxx/resourceGroups/v-xisuRG02"
    state: absent
'''
RETURN = '''
tag_info:
    description:
        - The tag info.
    returned: when I(scope=None)
    type: complex
    contains:
        id:
            description:
                - The ID of the tags wrapper resource.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Resources/tags/default"
        name:
            description:
                - The name of the tags wrapper resource.
            returned: always
            type: str
            sample: default
        type:
            description:
                - The type of the tags wrapper resource.
            returned: always
            type: str
            sample: Microsoft.Resources/tags
        properties:
            description:
                - The set of tags.
            returned: always
            type: dict
            sample: { 'tags': {'key1': 'value1', 'key2': 'value2'}}
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMTags(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            scope=dict(type='str', required=True),
            operation=dict(type='str', choices=['Replace', 'Merge', 'Delete'], default='Merge'),
            state=dict(type='str', default='present', choices=['present', 'absent']),
        )

        self.scope = None
        self.operation = None
        self.tags = None
        self.state = None

        self.results = dict(
            changed=False,
            tag_info=dict()
        )

        super(AzureRMTags, self).__init__(self.module_arg_spec,
                                          supports_tags=True,
                                          supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False
        if self.state == 'present':
            response = self.get_at_scope()
            if response and response['properties'].get('tags'):
                merge_tag, delete_tag, replace_tag = self.tags_update(response['properties']['tags'], self.tags)

                if (self.operation == 'Merge' and merge_tag) or (self.operation == 'Replace' and replace_tag) or (delete_tag and self.operation == 'Delete'):
                    changed = True
                    response = self.begin_update_at_scope(self.tags, self.operation)
            else:
                if self.tags:
                    changed = True
                    response = self.begin_create_or_update_at_scope(self.tags)
        else:
            response = self.get_at_scope()
            if response['properties']['tags']:
                changed = True
                response = self.delete_at_scope()

        self.results['changed'] = changed
        self.results['tag_info'] = response

        return self.results

    def begin_create_or_update_at_scope(self, tags):
        self.log('Creates or updates the entire set of tags on a resource or subscription.')
        try:
            response = self.rm_client.tags.begin_create_or_update_at_scope(self.scope,
                                                                           dict(properties=dict(tags=tags)))
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail('Creates or updates the entire set of tags on a resource or subscription got Exception as as {0}'.format(exc.message or str(exc)))
        return self.format_tags(response)

    def begin_update_at_scope(self, tags, operation):
        self.log('Selectively updates the set of tags on a resource or subscription.')
        try:
            response = self.rm_client.tags.begin_update_at_scope(self.scope,
                                                                 dict(operation=operation, properties=dict(tags=tags)))
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail('Selectively updates the set of tags on a resource or subscription got Excption as {0}'.format(exc.message or str(exc)))

        return self.format_tags(response)

    def delete_at_scope(self):
        self.log('Deletes the entire set of tags on a resource or subscription.')
        try:
            self.rm_client.tags.begin_delete_at_scope(self.scope)
        except Exception as exc:
            self.fail('Delete the entire set of tag got Excetion as {0}'.format(exc.message or str(exc)))

    def get_at_scope(self):
        self.log('Get properties for {0}'.format(self.scope))
        try:
            response = self.rm_client.tags.get_at_scope(self.scope)
            if response is not None:
                return self.format_tags(response)
        except Exception as exc:
            self.fail('Error when get the tags info under specified scope got Excetion as {0}'.format(exc.message or str(exc)))

    def format_tags(self, tags):
        results = dict(
            id=tags.id,
            name=tags.name,
            type=tags.type,
            properties=dict()
        )
        if tags.properties:
            results['properties'] = tags.properties.as_dict()

        return results

    def tags_update(self, old, new):
        old = old or dict()
        new = new or dict()
        merge_tag = not set(new.items()).issubset(set(old.items()))
        replace_tag = True
        if not merge_tag and len(old) == len(new):
            replace_tag = False
        delete_tag = False
        for key, value in new.items():
            if old.get(key) and old.get(key) == value:
                delete_tag = True
                break
        return merge_tag, delete_tag, replace_tag


def main():
    AzureRMTags()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: ec2_transit_gateway_info
short_description: Retrieve information about EC2 Transit Gateways in AWS
version_added: 1.0.0
version_added_collection: community.aws
description:
  - Gather information about EC2 Transit Gateways in AWS.
author:
  - "Bob Boldin (@BobBoldin)"
options:
  transit_gateway_ids:
    description:
      - A list of Transit Gateway IDs for which to gather information.
    aliases: [transit_gateway_id]
    type: list
    elements: str
    default: []
  filters:
    description:
      - A dictionary of filters to apply to the query. Each key-value pair represents a filter key and its corresponding value.
      - For a complete list of available filters,
        refer to the AWS documentation U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeTransitGateways.html).
    type: dict
    default: {}
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather info about all transit gateways
  amazon.aws.ec2_transit_gateway_info:

- name: Gather info about a particular transit gateway using filter transit gateway ID
  amazon.aws.ec2_transit_gateway_info:
    filters:
      transit-gateway-id: tgw-02c42332e6b7da829

- name: Gather info about a particular transit gateway using multiple option filters
  amazon.aws.ec2_transit_gateway_info:
    filters:
      options.dns-support: enable
      options.vpn-ecmp-support: enable

- name: Gather info about multiple transit gateways using module param
  amazon.aws.ec2_transit_gateway_info:
    transit_gateway_ids:
      - tgw-02c42332e6b7da829
      - tgw-03c53443d5a8cb716
"""

RETURN = r"""
transit_gateways:
    description:
      - Transit gateways that match the provided filters.
      - Each element consists of a dict with all the information related to that transit gateway.
    returned: on success
    type: list
    elements: dict
    contains:
        creation_time:
            description: The creation time.
            returned: always
            type: str
            sample: "2019-02-05T16:19:58+00:00"
        description:
            description: The description of the transit gateway.
            returned: always
            type: str
            sample: "A transit gateway"
        options:
            description: A dictionary of the transit gateway options.
            returned: always
            type: dict
            contains:
                 amazon_side_asn:
                    description:
                      - A private Autonomous System Number (ASN) for the Amazon ide of a BGP session.
                      - The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.
                    returned: always
                    type: int
                    sample: 64512
                 auto_accept_shared_attachments:
                    description: Indicates whether attachment requests are automatically accepted.
                    returned: always
                    type: str
                    sample: "enable"
                 default_route_table_association:
                    description: Indicates whether resource attachments are automatically associated with the default association route table.
                    returned: always
                    type: str
                    sample: "disable"
                 association_default_route_table_id:
                    description: The ID of the default association route table.
                    returned: when present
                    type: str
                    sample: "tgw-rtb-0fd332c911223344"
                 default_route_table_propagation:
                    description: Indicates whether resource attachments automatically propagate routes to the default propagation route table.
                    returned: always
                    type: str
                    sample: "disable"
                 dns_support:
                    description: Indicates whether DNS support is enabled.
                    returned: always
                    type: str
                    sample: "enable"
                 multicast_support:
                    description: Indicates whether Multicast support is enabled.
                    returned: always
                    type: str
                    sample: "enable"
                    version_added: 7.3.0
                 propagation_default_route_table_id:
                    description: The ID of the default propagation route table.
                    returned: when present
                    type: str
                    sample: "rtb-11223344"
                 vpn_ecmp_support:
                    description: Indicates whether Equal Cost Multipath Protocol support is enabled.
                    returned: always
                    type: str
                    sample: "enable"
        owner_id:
            description: The AWS account number ID which owns the transit gateway.
            returned: always
            type: str
            sample: "123456789012"
        state:
            description: The state of the transit gateway.
            returned: always
            type: str
            sample: "available"
        tags:
            description: A dict of tags associated with the transit gateway.
            returned: always
            type: dict
            sample: {
                      "Name": "A sample TGW",
                      "Env": "Dev"
                    }
        transit_gateway_arn:
            description: The Amazon Resource Name (ARN) of the transit gateway.
            returned: always
            type: str
            sample: "arn:aws:ec2:us-west-2:123456789012:transit-gateway/tgw-02c42332e6b7da829"
        transit_gateway_id:
            description: The ID of the transit gateway.
            returned: always
            type: str
            sample: "tgw-02c42332e6b7da829"
"""

from typing import Any
from typing import Dict
from typing import List

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_transit_gateways
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def get_transit_gateway_response(module: AnsibleAWSModule, connection) -> Dict[str, Any]:
    """
    Get transit gateway response from AWS.

    module     : AnsibleAWSModule object
    connection : boto3 client connection object
    :return: Response from describe_transit_gateways call
    """
    filters = ansible_dict_to_boto3_filter_list(module.params["filters"])
    transit_gateway_ids = module.params["transit_gateway_ids"]

    params = {}
    if transit_gateway_ids:
        params["TransitGatewayIds"] = transit_gateway_ids
    if filters:
        params["Filters"] = filters

    result = describe_ec2_transit_gateways(connection, **params)
    return result


def extract_transit_gateway_info(transit_gateway: Dict[str, Any]) -> Dict[str, Any]:
    """
    Extract and transform transit gateway information.

    transit_gateway : The transit gateway data from AWS
    :return: Transformed transit gateway information
    """
    tgw_data = camel_dict_to_snake_dict(transit_gateway, ignore_list=["Tags"])
    tgw_data["tags"] = boto3_tag_list_to_ansible_dict(transit_gateway.get("Tags", []))
    return tgw_data


def describe_transit_gateways(module: AnsibleAWSModule, connection) -> List[Dict[str, Any]]:
    """
    Describe transit gateways.

    module     : AnsibleAWSModule object
    connection : boto3 client connection object
    :return: List of transit gateways
    """
    response = get_transit_gateway_response(module, connection)
    return [extract_transit_gateway_info(tgw) for tgw in response]


def setup_module_object() -> AnsibleAWSModule:
    """
    Merge argument spec and create Ansible module object.
    :return: Ansible module object
    """
    argument_spec = dict(
        transit_gateway_ids=dict(type="list", default=[], elements="str", aliases=["transit_gateway_id"]),
        filters=dict(type="dict", default={}),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    return module


def main():
    module = setup_module_object()
    results = {"changed": False}

    connection = module.client("ec2")
    try:
        transit_gateways = describe_transit_gateways(module, connection)
        results["transit_gateways"] = transit_gateways
    except AnsibleEC2Error as e:
        module.fail_json_aws(e)

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Prasoon Karunan V (@prasoonkarunan)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_auto_logon
short_description: Adds or Sets auto logon registry keys.
description:
  - Used to apply auto logon registry setting.
version_added: 2.7.0
options:
  logon_count:
    description:
      - The number of times to do an automatic logon.
      - This count is deremented by Windows everytime an automatic logon is
        performed.
      - Once the count reaches C(0) then the automatic logon process is
        disabled.
    type: int
  username:
    description:
      - Username to login automatically.
      - Must be set when C(state=present).
      - This can be the Netlogon or UPN of a domain account and is
        automatically parsed to the C(DefaultUserName) and C(DefaultDomainName)
        registry properties.
    type: str
  password:
    description:
      - Password to be used for automatic login.
      - Must be set when C(state=present).
      - Value of this input will be used as password for I(username).
      - While this value is encrypted by LSA it is decryptable to any user who
        is an Administrator on the remote host.
    type: str
  state:
    description:
      - Whether the registry key should be C(present) or C(absent).
    type: str
    choices: [ absent, present ]
    default: present
author:
  - Prasoon Karunan V (@prasoonkarunan)
'''

EXAMPLES = r'''
- name: Set autologon for user1
  ansible.windows.win_auto_logon:
    username: User1
    password: str0ngp@ssword

- name: Set autologon for abc.com\user1
  ansible.windows.win_auto_logon:
    username: abc.com\User1
    password: str0ngp@ssword

- name: Remove autologon for user1
  ansible.windows.win_auto_logon:
    state: absent

- name: Set autologon for user1 with a limited logon count
  ansible.windows.win_auto_logon:
    username: User1
    password: str0ngp@ssword
    logon_count: 5
'''

RETURN = r'''
#
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: ucs_service_profile_from_template
short_description: Configures Service Profiles from templates on Cisco UCS Manager
description:
- Configures Service Profile created from templates on Cisco UCS Manager.
extends_documentation_fragment: cisco.ucs.ucs
options:
  state:
    description:
    - If C(present), will verify Service Profiles are present and will create if needed.
    - If C(absent), will verify Service Profiles are absent and will delete if needed.
    choices: [present, absent]
    default: present
    type: str
  name:
    description:
    - The name of the service profile.
    - This name can be between 2 and 32 alphanumeric characters.
    - "You cannot use spaces or any special characters other than - (hyphen), \"_\" (underscore), : (colon), and . (period)."
    - This name must be unique across all service profiles and service profile templates within the same organization.
    required: true
    type: str
  source_template:
    description:
    - The name of the service profile template used to create this serivce profile.
    required: true
    type: str
  power_state:
    description:
    - The power state to be applied when this service profile is associated with a server.
    - If no value is provided, the power_state for the service profile will not be modified.
    choices: [up, down]
    type: str
  user_label:
    description:
    - The User Label you want to assign to this service profile.
    type: str
  org_dn:
    description:
    - Org dn (distinguished name)
    default: org-root
    type: str
  description:
    description:
    - Optional
    - The Description of the service profile
    type: str
requirements:
- ucsmsdk
author:
- David Soper (@dsoper2)
- CiscoUcs (@CiscoUcs)
'''

EXAMPLES = r'''
- name: Configure Service Profile from Template
  cisco.ucs.ucs_service_profile_from_template:
    hostname: 172.16.143.150
    username: admin
    password: password
    name: test-sp-instance1
    source_template: test-sp
    discription: Created from Ansible

- name: Remove Service Profile
  cisco.ucs.ucs_service_profile_from_template:
    hostname: 172.16.143.150
    username: admin
    password: password
    name: test-sp-instance1
    state: absent
'''

RETURN = r'''
#
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.ucs.plugins.module_utils.ucs import UCSModule, ucs_argument_spec


def main():
    argument_spec = ucs_argument_spec.copy()
    argument_spec.update(
        org_dn=dict(type='str', default='org-root'),
        name=dict(type='str', required=True),
        source_template=dict(type='str', required=True),
        user_label=dict(type='str'),
        power_state=dict(type='str', choices=['up', 'down']),
        state=dict(type='str', default='present', choices=['present', 'absent']),
        description=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    ucs = UCSModule(module)

    err = False

    # UCSModule creation above verifies ucsmsdk is present and exits on failure.  Additional imports are done below.
    from ucsmsdk.mometa.ls.LsServer import LsServer
    from ucsmsdk.mometa.ls.LsPower import LsPower

    changed = False
    try:
        mo_exists = False
        props_match = False
        dn = module.params['org_dn'] + '/ls-' + module.params['name']

        mo = ucs.login_handle.query_dn(dn)
        if mo:
            mo_exists = True

        if module.params['state'] == 'absent':
            # mo must exist but all properties do not have to match
            if mo_exists:
                if not module.check_mode:
                    ucs.login_handle.remove_mo(mo)
                    ucs.login_handle.commit()
                changed = True
        else:
            if mo_exists:
                # check top-level mo props
                kwargs = dict(src_templ_name=module.params['source_template'])
                kwargs['usr_lbl'] = module.params['user_label']
                # service profiles are of type 'instance'
                kwargs['type'] = 'instance'

                if mo.check_prop_match(**kwargs):
                    # top-level props match
                    if module.params.get('power_state'):
                        child_dn = dn + '/power'
                        mo_1 = ucs.login_handle.query_dn(child_dn)
                        if mo_1:
                            kwargs = dict(state=module.params['power_state'])
                            if mo_1.check_prop_match(**kwargs):
                                props_match = True
                    else:
                        # no power state provided, use existing state as match
                        props_match = True

            if not props_match:
                if not module.check_mode:
                    # create if mo does not already exist
                    mo = LsServer(
                        parent_mo_or_dn=module.params['org_dn'],
                        name=module.params['name'],
                        src_templ_name=module.params['source_template'],
                        type='instance',
                        usr_lbl=module.params['user_label'],
                        descr=module.params['description'],
                    )
                    if module.params.get('power_state'):
                        admin_state = 'admin-' + module.params['power_state']
                        mo_1 = LsPower(
                            parent_mo_or_dn=mo,
                            state=admin_state,
                        )

                    ucs.login_handle.add_mo(mo, True)
                    ucs.login_handle.commit()
                changed = True

    except Exception as e:
        err = True
        ucs.result['msg'] = "setup error: %s " % str(e)

    ucs.result['changed'] = changed
    if err:
        module.fail_json(**ucs.result)
    module.exit_json(**ucs.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2025 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_diskaccess
version_added: '3.3.0'
short_description: Show the details for the disk access
description:
    - Get or list the details for the disk access.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: true
    name:
        description:
            - The disk access name.
        required: true
        type: str
    location:
        description:
            - Resource location.
        type: str
    extended_location:
        description:
            - The extended location where the disk access will be created.
        type: dict
        suboptions:
            name:
                description:
                    - The name of the extended location.
                type: str
                required: true
            type:
                description:
                    - The type of the extended location.
                type: str
                default: EdgeZone
                choices:
                    - EdgeZone
    state:
        description:
            - State of the disk access.
            - Use C(present) to create or update a disk access and use C(absent) to delete.
        type: str
        default: present
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a new disk access
  azure_rm_diskaccess:
    resource_group: myRG
    name: testaccess
    location: westus2

- name: Create a new disk access
  azure_rm_diskaccess:
    resource_group: myRG
    name: testaccess
    location: westus
    extended_location:
      name: losangeles
      type: EdgeZone
    tags:
      key1: value1

- name: Delete the disk access
  azure_rm_diskaccess:
    resource_group: myRG
    name: testaccess
    state: absent
'''

RETURN = '''
state:
    description:
        - Details for the disk access.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            type: str
            returned: always
            sample:
        resource_group:
            description:
                - Resource group name.
            type: str
            returned: always
            sample: myRG
        name:
            description:
                - Resource name.
            type: str
            returned: always
            sample: testaccess
        type:
            description:
                - Resource Type.
            type: str
            returned: always
            sample: "Microsoft.Compute/diskAccesses"
        tags:
            description:
                - The tags of the disk access.
            type: dict
            returned: always
            sample: {key1: value1, key2: value2}
        provisioning_state:
            description:
                - The disk access resource provisioning state.
            type: str
            returned: always
            sample: Successed
        extended_location:
            description:
                - The extended location where the disk access will be created.
            type: dict
            returned: always
            sample: {"name": "portland", "type": "EdgeZone"}
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    pass


class AzureRMDiskAccess(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            extended_location=dict(
                type='dict',
                options=dict(
                    name=dict(type='str', required=True),
                    type=dict(type='str', default='EdgeZone', choices=['EdgeZone']),
                )
            ),
            state=dict(
                type='str',
                choices=['present', 'absent'],
                default='present'
            )
        )
        # store the results of the module operation
        self.results = dict()
        self.resource_group = None
        self.name = None
        self.location = None
        self.extended_location = None
        self.state = None

        super(AzureRMDiskAccess, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True, facts_module=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec) + ['tags']:
            setattr(self, key, kwargs[key])

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        changed = False
        response = self.get_diskaccess()

        if self.state == 'present':
            if response is not None:
                if self.extended_location is not None and self.extended_location.get('name') != response['extended_location'].get('name'):
                    self.fail("Extended location cannot be changed")

                update_tags, self.tags = self.update_tags(response.get('tags'))
                if update_tags:
                    changed = True
                    if not self.check_mode:
                        response = self.update_diskaccess()
            else:
                changed = True
                if not self.check_mode:
                    response = self.create_or_update()
        else:
            if not self.check_mode:
                if response is not None:
                    response = self.delete_diskaccess()
                    changed = True
                else:
                    changed = False
            else:
                changed = True

        self.results['changed'] = changed
        self.results['state'] = response

        return self.results

    def get_diskaccess(self):
        try:
            return self.to_dict(self.disk_client.disk_accesses.get(self.resource_group, self.name))
        except ResourceNotFoundError:
            pass

    def create_or_update(self):
        try:
            poller = self.disk_client.disk_accesses.begin_create_or_update(self.resource_group, self.name, dict(tags=self.tags,
                                                                                                                location=self.location,
                                                                                                                extended_location=self.extended_location))
            if isinstance(poller, LROPoller):
                response = self.get_poller_result(poller)
                return self.to_dict(response)

        except Exception as ec:
            self.fail('Error when create disk access {0}: {1}'.format(self.name, ec))

    def update_diskaccess(self):
        try:
            poller = self.disk_client.disk_accesses.begin_update(self.resource_group, self.name, dict(tags=self.tags))
            if isinstance(poller, LROPoller):
                response = self.get_poller_result(poller)
                return self.to_dict(response)
        except Exception as ec:
            self.fail('Error when update disk access {0}: {1}'.format(self.name, ec))

    def delete_diskaccess(self):
        try:
            self.disk_client.disk_accesses.begin_delete(self.resource_group, self.name)
        except Exception as ec:
            self.fail('Error when delete disk access {0}: {1}'.format(self.name, ec))

    def get_diskaccess(self):
        try:
            return self.to_dict(self.disk_client.disk_accesses.get(self.resource_group, self.name))
        except ResourceNotFoundError:
            pass

    def to_dict(self, item):
        if item is None:
            return None
        disk_access = dict(
            resource_group=self.resource_group,
            name=item.name,
            id=item.id,
            type=item.type,
            location=item.location,
            provisioning_state=item.provisioning_state,
            extended_location=dict(),
            tags=item.tags
        )
        if item.extended_location is not None:
            disk_access['extended_location']['name'] = item.extended_location.name
            disk_access['extended_location']['type'] = item.extended_location.type
        return disk_access


def main():
    AzureRMDiskAccess()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2017, Kairo Araujo <kairo@kairo.eti.br>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Kairo Araujo (@kairoaraujo)
module: aix_filesystem
short_description: Configure LVM and NFS file systems for AIX
description:
  - This module creates, removes, mount and unmount LVM and NFS file system for AIX using C(/etc/filesystems).
  - For LVM file systems is possible to resize a file system.
deprecated:
  removed_in: 15.0.0
  why: The module is not actively maintained.
  alternative: >-
    Use C(ibm.power_aix.filesystem) instead.
    See U(https://ibm.github.io/ansible-power-aix/modules/filesystem.html) for details.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  account_subsystem:
    description:
      - Specifies whether the file system is to be processed by the accounting subsystem.
    type: bool
    default: false
  attributes:
    description:
      - Specifies attributes for files system separated by comma.
    type: list
    elements: str
    default:
      - agblksize=4096
      - isnapshot=no
  auto_mount:
    description:
      - File system is automatically mounted at system restart.
    type: bool
    default: true
  device:
    description:
      - Logical volume (LV) device name or remote export device to create a NFS file system.
      - It is used to create a file system on an already existing logical volume or the exported NFS file system.
      - If not mentioned a new logical volume name is created following AIX standards (LVM).
    type: str
  fs_type:
    description:
      - Specifies the virtual file system type.
    type: str
    default: jfs2
  permissions:
    description:
      - Set file system permissions. V(rw) (read-write) or V(ro) (read-only).
    type: str
    choices: [ro, rw]
    default: rw
  mount_group:
    description:
      - Specifies the mount group.
    type: str
  filesystem:
    description:
      - Specifies the mount point, which is the directory where the file system will be mounted.
    type: str
    required: true
  nfs_server:
    description:
      - Specifies a Network File System (NFS) server.
    type: str
  rm_mount_point:
    description:
      - Removes the mount point directory when used with state V(absent).
    type: bool
    default: false
  size:
    description:
      - Specifies the file system size.
      - For already present it resizes the filesystem.
      - 512-byte blocks, megabytes or gigabytes. If the value has M specified it is in megabytes. If the value has G specified
        it is in gigabytes.
      - If no M or G the value is 512-byte blocks.
      - If V(+) is specified in begin of value, the value is added.
      - If V(-) is specified in begin of value, the value is removed.
      - If neither V(+) nor V(-) is specified, then the total value is the specified.
      - Size respects the LVM AIX standards.
    type: str
  state:
    description:
      - Controls the file system state.
      - V(present) check if file system exists, creates or resize.
      - V(absent) removes existing file system if already V(unmounted).
      - V(mounted) checks if the file system is mounted or mount the file system.
      - V(unmounted) check if the file system is unmounted or unmount the file system.
    type: str
    choices: [absent, mounted, present, unmounted]
    default: present
  vg:
    description:
      - Specifies an existing volume group (VG).
    type: str
notes:
  - For more O(attributes), please check C(crfs) AIX manual.
"""

EXAMPLES = r"""
- name: Create filesystem in a previously defined logical volume.
  community.general.aix_filesystem:
    device: testlv
    filesystem: /testfs
    state: present

- name: Creating NFS filesystem from nfshost.
  community.general.aix_filesystem:
    device: /home/ftp
    nfs_server: nfshost
    filesystem: /home/ftp
    state: present

- name: Creating a new file system without a previously logical volume.
  community.general.aix_filesystem:
    filesystem: /newfs
    size: 1G
    state: present
    vg: datavg

- name: Unmounting /testfs.
  community.general.aix_filesystem:
    filesystem: /testfs
    state: unmounted

- name: Resizing /mksysb to +512M.
  community.general.aix_filesystem:
    filesystem: /mksysb
    size: +512M
    state: present

- name: Resizing /mksysb to 11G.
  community.general.aix_filesystem:
    filesystem: /mksysb
    size: 11G
    state: present

- name: Resizing /mksysb to -2G.
  community.general.aix_filesystem:
    filesystem: /mksysb
    size: -2G
    state: present

- name: Remove NFS filesystem /home/ftp.
  community.general.aix_filesystem:
    filesystem: /home/ftp
    rm_mount_point: true
    state: absent

- name: Remove /newfs.
  community.general.aix_filesystem:
    filesystem: /newfs
    rm_mount_point: true
    state: absent
"""


import re
from os.path import ismount

from ansible.module_utils.basic import AnsibleModule


def _fs_exists(module, filesystem):
    """
    Check if file system already exists on /etc/filesystems.

    :param module: Ansible module.
    :param community.general.filesystem: filesystem name.
    :return: True or False.
    """
    lsfs_cmd = module.get_bin_path("lsfs", True)
    rc, lsfs_out, err = module.run_command([lsfs_cmd, "-l", filesystem])
    if rc == 1:
        if re.findall("No record matching", err):
            return False

        else:
            module.fail_json(msg=f"Failed to run lsfs. Error message: {err}")

    else:
        return True


def _check_nfs_device(module, nfs_host, device):
    """
    Validate if NFS server is exporting the device (remote export).

    :param module: Ansible module.
    :param nfs_host: nfs_host parameter, NFS server.
    :param device: device parameter, remote export.
    :return: True or False.
    """
    showmount_cmd = module.get_bin_path("showmount", True)
    rc, showmount_out, err = module.run_command([showmount_cmd, "-a", nfs_host])
    if rc != 0:
        module.fail_json(msg=f"Failed to run showmount. Error message: {err}")
    else:
        showmount_data = showmount_out.splitlines()
        return any(line.split(":")[1] == device for line in showmount_data)


def _validate_vg(module, vg):
    """
    Check the current state of volume group.

    :param module: Ansible module argument spec.
    :param vg: Volume Group name.
    :return: True (VG in varyon state) or False (VG in varyoff state) or
             None (VG does not exist), message.
    """
    lsvg_cmd = module.get_bin_path("lsvg", True)
    rc, current_active_vgs, err = module.run_command([lsvg_cmd, "-o"])
    if rc != 0:
        module.fail_json(msg=f"Failed executing {lsvg_cmd} command.")

    rc, current_all_vgs, err = module.run_command([lsvg_cmd])
    if rc != 0:
        module.fail_json(msg=f"Failed executing {lsvg_cmd} command.")

    if vg in current_all_vgs and vg not in current_active_vgs:
        msg = f"Volume group {vg} is in varyoff state."
        return False, msg
    elif vg in current_active_vgs:
        msg = f"Volume group {vg} is in varyon state."
        return True, msg
    else:
        msg = f"Volume group {vg} does not exist."
        return None, msg


def resize_fs(module, filesystem, size):
    """Resize LVM file system."""

    chfs_cmd = module.get_bin_path("chfs", True)
    if not module.check_mode:
        rc, chfs_out, err = module.run_command([chfs_cmd, "-a", f"size={size}", filesystem])

        if rc == 28:
            changed = False
            return changed, chfs_out
        elif rc != 0:
            if re.findall("Maximum allocation for logical", err):
                changed = False
                return changed, err
            else:
                module.fail_json(msg=f"Failed to run chfs. Error message: {err}")

        else:
            if re.findall("The filesystem size is already", chfs_out):
                changed = False
            else:
                changed = True

            return changed, chfs_out
    else:
        changed = True
        msg = ""

        return changed, msg


def create_fs(
    module,
    fs_type,
    filesystem,
    vg,
    device,
    size,
    mount_group,
    auto_mount,
    account_subsystem,
    permissions,
    nfs_server,
    attributes,
):
    """Create LVM file system or NFS remote mount point."""

    attributes = " -a ".join(attributes)

    # Parameters definition.
    account_subsys_opt = {True: "-t yes", False: "-t no"}

    if nfs_server is not None:
        auto_mount_opt = {True: "-A", False: "-a"}

    else:
        auto_mount_opt = {True: "-A yes", False: "-A no"}

    if size is None:
        size = ""
    else:
        size = f"-a size={size}"

    if device is None:
        device = ""
    else:
        device = f"-d {device}"

    if vg is None:
        vg = ""
    else:
        vg_state, msg = _validate_vg(module, vg)
        if vg_state:
            vg = f"-g {vg}"
        else:
            changed = False

            return changed, msg

    if mount_group is None:
        mount_group = ""

    else:
        mount_group = f"-u {mount_group}"

    auto_mount = auto_mount_opt[auto_mount]
    account_subsystem = account_subsys_opt[account_subsystem]

    if nfs_server is not None:
        # Creates a NFS file system.
        mknfsmnt_cmd = module.get_bin_path("mknfsmnt", True)
        if not module.check_mode:
            rc, mknfsmnt_out, err = module.run_command(
                [mknfsmnt_cmd, "-f", filesystem, device, "-h", nfs_server, "-t", permissions, auto_mount, "-w", "bg"]
            )
            if rc != 0:
                module.fail_json(msg=f"Failed to run mknfsmnt. Error message: {err}")
            else:
                changed = True
                msg = f"NFS file system {filesystem} created."

                return changed, msg
        else:
            changed = True
            msg = ""

            return changed, msg

    else:
        # Creates a LVM file system.
        crfs_cmd = module.get_bin_path("crfs", True)
        if not module.check_mode:
            cmd = [crfs_cmd]

            cmd.append("-v")
            cmd.append(fs_type)

            if vg:
                (flag, value) = vg.split()
                cmd.append(flag)
                cmd.append(value)

            if device:
                (flag, value) = device.split()
                cmd.append(flag)
                cmd.append(value)

            cmd.append("-m")
            cmd.append(filesystem)

            if mount_group:
                (flag, value) = mount_group.split()
                cmd.append(flag)
                cmd.append(value)

            if auto_mount:
                (flag, value) = auto_mount.split()
                cmd.append(flag)
                cmd.append(value)

            if account_subsystem:
                (flag, value) = account_subsystem.split()
                cmd.append(flag)
                cmd.append(value)

            cmd.append("-p")
            cmd.append(permissions)

            if size:
                (flag, value) = size.split()
                cmd.append(flag)
                cmd.append(value)

            if attributes:
                splitted_attributes = attributes.split()
                cmd.append("-a")
                for value in splitted_attributes:
                    cmd.append(value)

            rc, crfs_out, err = module.run_command(cmd)

            if rc == 10:
                module.exit_json(
                    msg=f"Using a existent previously defined logical volume, volume group needs to be empty. {err}"
                )

            elif rc != 0:
                module.fail_json(msg=f"Failed to run {cmd}. Error message: {err}")

            else:
                changed = True
                return changed, crfs_out
        else:
            changed = True
            msg = ""

            return changed, msg


def remove_fs(module, filesystem, rm_mount_point):
    """Remove an LVM file system or NFS entry."""

    # Command parameters.
    rm_mount_point_opt = {True: "-r", False: ""}

    rm_mount_point = rm_mount_point_opt[rm_mount_point]

    rmfs_cmd = module.get_bin_path("rmfs", True)
    if not module.check_mode:
        cmd = [rmfs_cmd, "-r", rm_mount_point, filesystem]
        rc, rmfs_out, err = module.run_command(cmd)
        if rc != 0:
            module.fail_json(msg=f"Failed to run {cmd}. Error message: {err}")
        else:
            changed = True
            msg = rmfs_out
            if not rmfs_out:
                msg = f"File system {filesystem} removed."

            return changed, msg
    else:
        changed = True
        msg = ""

        return changed, msg


def mount_fs(module, filesystem):
    """Mount a file system."""
    mount_cmd = module.get_bin_path("mount", True)

    if not module.check_mode:
        rc, mount_out, err = module.run_command([mount_cmd, filesystem])
        if rc != 0:
            module.fail_json(msg=f"Failed to run mount. Error message: {err}")
        else:
            changed = True
            msg = f"File system {filesystem} mounted."

            return changed, msg
    else:
        changed = True
        msg = ""

        return changed, msg


def unmount_fs(module, filesystem):
    """Unmount a file system."""
    unmount_cmd = module.get_bin_path("unmount", True)

    if not module.check_mode:
        rc, unmount_out, err = module.run_command([unmount_cmd, filesystem])
        if rc != 0:
            module.fail_json(msg=f"Failed to run unmount. Error message: {err}")
        else:
            changed = True
            msg = f"File system {filesystem} unmounted."

            return changed, msg
    else:
        changed = True
        msg = ""

        return changed, msg


def main():
    module = AnsibleModule(
        argument_spec=dict(
            account_subsystem=dict(type="bool", default=False),
            attributes=dict(type="list", elements="str", default=["agblksize=4096", "isnapshot=no"]),
            auto_mount=dict(type="bool", default=True),
            device=dict(type="str"),
            filesystem=dict(type="str", required=True),
            fs_type=dict(type="str", default="jfs2"),
            permissions=dict(type="str", default="rw", choices=["rw", "ro"]),
            mount_group=dict(type="str"),
            nfs_server=dict(type="str"),
            rm_mount_point=dict(type="bool", default=False),
            size=dict(type="str"),
            state=dict(type="str", default="present", choices=["absent", "mounted", "present", "unmounted"]),
            vg=dict(type="str"),
        ),
        supports_check_mode=True,
    )

    account_subsystem = module.params["account_subsystem"]
    attributes = module.params["attributes"]
    auto_mount = module.params["auto_mount"]
    device = module.params["device"]
    fs_type = module.params["fs_type"]
    permissions = module.params["permissions"]
    mount_group = module.params["mount_group"]
    filesystem = module.params["filesystem"]
    nfs_server = module.params["nfs_server"]
    rm_mount_point = module.params["rm_mount_point"]
    size = module.params["size"]
    state = module.params["state"]
    vg = module.params["vg"]

    result = dict(
        changed=False,
        msg="",
    )

    if state == "present":
        fs_mounted = ismount(filesystem)
        fs_exists = _fs_exists(module, filesystem)

        # Check if fs is mounted or exists.
        if fs_mounted or fs_exists:
            result["msg"] = f"File system {filesystem} already exists."
            result["changed"] = False

            # If parameter size was passed, resize fs.
            if size is not None:
                result["changed"], result["msg"] = resize_fs(module, filesystem, size)

        # If fs doesn't exist, create it.
        else:
            # Check if fs will be a NFS device.
            if nfs_server is not None:
                if device is None:
                    result["msg"] = 'Parameter "device" is required when "nfs_server" is defined.'
                    module.fail_json(**result)
                else:
                    # Create a fs from NFS export.
                    if _check_nfs_device(module, nfs_server, device):
                        result["changed"], result["msg"] = create_fs(
                            module,
                            fs_type,
                            filesystem,
                            vg,
                            device,
                            size,
                            mount_group,
                            auto_mount,
                            account_subsystem,
                            permissions,
                            nfs_server,
                            attributes,
                        )

            if device is None:
                if vg is None:
                    result["msg"] = 'Required parameter "device" and/or "vg" is missing for filesystem creation.'
                    module.fail_json(**result)
                else:
                    # Create a fs from
                    result["changed"], result["msg"] = create_fs(
                        module,
                        fs_type,
                        filesystem,
                        vg,
                        device,
                        size,
                        mount_group,
                        auto_mount,
                        account_subsystem,
                        permissions,
                        nfs_server,
                        attributes,
                    )

            if device is not None and nfs_server is None:
                # Create a fs from a previously lv device.
                result["changed"], result["msg"] = create_fs(
                    module,
                    fs_type,
                    filesystem,
                    vg,
                    device,
                    size,
                    mount_group,
                    auto_mount,
                    account_subsystem,
                    permissions,
                    nfs_server,
                    attributes,
                )

    elif state == "absent":
        if ismount(filesystem):
            result["msg"] = f"File system {filesystem} mounted."

        else:
            fs_status = _fs_exists(module, filesystem)
            if not fs_status:
                result["msg"] = f"File system {filesystem} does not exist."
            else:
                result["changed"], result["msg"] = remove_fs(module, filesystem, rm_mount_point)

    elif state == "mounted":
        if ismount(filesystem):
            result["changed"] = False
            result["msg"] = f"File system {filesystem} already mounted."
        else:
            result["changed"], result["msg"] = mount_fs(module, filesystem)

    elif state == "unmounted":
        if not ismount(filesystem):
            result["changed"] = False
            result["msg"] = f"File system {filesystem} already unmounted."
        else:
            result["changed"], result["msg"] = unmount_fs(module, filesystem)

    else:
        # Unreachable codeblock
        result["msg"] = f"Unexpected state {state}."
        module.fail_json(**result)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_firmware_upgrades_by_device_info
short_description: Information module for organizations _firmware _upgrades _by _device
description:
  - Get all organizations _firmware _upgrades _by _device. - > Get firmware upgrade
    status for the filtered devices. This endpoint currently only supports Meraki
    switches and access points.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 50.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter by network.
    elements: str
    type: list
  serials:
    description:
      - >
        Serials query parameter. Optional parameter to filter by serial number. All
        returned devices will have a serial number that is an exact match.
    elements: str
    type: list
  macs:
    description:
      - >
        Macs query parameter. Optional parameter to filter by one or more MAC addresses
        belonging to devices. All devices returned belong to MAC addresses that are
        an exact match.
    elements: str
    type: list
  firmwareUpgradeBatchIds:
    description:
      - FirmwareUpgradeBatchIds query parameter. Optional parameter to filter by firmware
        upgrade batch ids.
    elements: str
    type: list
  upgradeStatuses:
    description:
      - UpgradeStatuses query parameter. Optional parameter to filter by firmware
        upgrade statuses.
    elements: str
    type: list
  currentUpgradesOnly:
    description:
      - CurrentUpgradesOnly query parameter. Optional parameter to filter to only
        current or pending upgrade statuses.
    type: bool
  limitPerDevice:
    description:
      - >
        LimitPerDevice query parameter. Optional parameter to limit the number of
        upgrade statuses returned per device. If omitted, a value of 5 is used.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationFirmwareUpgradesByDevice
    description: Complete reference of the getOrganizationFirmwareUpgradesByDevice
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-firmware-upgrades-by-device
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_firmware_upgrades_by_device,
  - Paths used are
    get /organizations/{organizationId}/firmware/upgrades/byDevice,
"""

EXAMPLES = r"""
- name: Get all organizations _firmware _upgrades _by _device
  cisco.meraki.organizations_firmware_upgrades_by_device_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    serials: []
    macs: []
    firmwareUpgradeBatchIds: []
    upgradeStatuses: []
    currentUpgradesOnly: true
    limitPerDevice: 0
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "serial": "string",
        "name": "string",
        "deviceStatus": "string",
        "checkinFinishedAt": {},
        "checkinStartedAt": {},
        "detailedStatus": {},
        "downloadFinishedAt": {},
        "downloadStartedAt": {},
        "downloadStatus": {},
        "installFinishedAt": {},
        "installStartedAt": {},
        "installStatus": {},
        "verifyFinishedAt": {},
        "verifyStartedAt": {},
        "verifyStatus": {},
        "upgrade": {
          "time": "string",
          "fromVersion": {
            "id": "string",
            "shortName": "string",
            "releaseDate": "string"
          },
          "toVersion": {
            "id": "string",
            "shortName": "string",
            "releaseDate": "string"
          },
          "status": "string",
          "id": "string",
          "upgradeBatchId": "string",
          "staged": {
            "group": {
              "id": "string"
            }
          }
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_node_setting
short_description: Manage Fabric Policy Node Settings on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Fabric Policy Node Settings on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Fabric Policy template.
    type: str
    required: true
    aliases: [ fabric_template ]
  name:
    description:
    - The name of the Fabric Policy Node Settings.
    type: str
    aliases: [ node_setting ]
  uuid:
    description:
    - The UUID of the Fabric Policy Node Settings.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ node_setting_uuid ]
  description:
    description:
    - The description of the Fabric Policy Node Settings.
    type: str
  synce:
    description:
    - The Synchronous Ethernet (SyncE) Interface Policy of the Fabric Policy Node Settings.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the SyncE Interface Policy.
        - Both O(synce.admin_state) and O(synce.quality_level) are required when the state is C(enabled).
        - Use C(disabled) to remove the SyncE Interface Policy.
        type: str
        choices: [ enabled, disabled ]
      admin_state:
        description:
        - The administrative state of the SyncE Interface Policy.
        type: str
        choices: [ enabled, disabled ]
      quality_level:
        description:
        - The quality level option of the SyncE Interface Policy.
        type: str
        choices: [ option_1, option_2_generation_1, option_2_generation_2 ]
  ptp:
    description:
    - The Precision Time Protocol (PTP) of the Fabric Policy Node Settings.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the PTP Settings.
        - Both O(ptp.node_domain) and O(ptp.priority_2) are required when the state is C(enabled).
        - Use C(disabled) to remove the PTP Settings.
        type: str
        choices: [ enabled, disabled ]
      node_domain:
        description:
        - The node domain number of the PTP Settings.
        - The value must be between 24 and 43.
        type: int
      priority_2:
        description:
        - The value that is used when advertising this clock.
        - The value must be between 0 and 255, lower values are prioritized.
        - The PTP priority 1 is set to a fixed value of 128.
        type: int
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Fabric Policy template.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new Fabric Policy Node Setting
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: ns1
    state: present
  register: create_ns1

- name: Add SyncE config to the existing Fabric Policy Node Setting using Name
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: ns1
    synce:
      admin_state: enabled
      quality_level: option_2_generation_1
    state: present

- name: Add PTP config to the existing Fabric Policy Node Setting using UUID
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    uuid: "{{ create_ns1.current.uuid }}"
    name: ns1_updated
    ptp:
      node_domain: 25
      priority_2: 100
    state: present

- name: Query an existing Fabric Policy Node Setting using UUID
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    uuid: "{{ create_ns1.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query an existing Fabric Policy Node Setting using Name
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: ns1_updated
    state: query
  register: query_with_name

- name: Query all Fabric Policy Node Setting
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    state: query
  register: query_all

- name: Delete an existing Fabric Policy Node Setting using UUID
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    uuid: "{{ create_ns1.current.uuid }}"
    state: absent

- name: Delete an existing Fabric Policy Node Setting using Name
  cisco.mso.ndo_node_setting:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: fabric_template
    name: ns1_updated
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.constants import SYNC_E_QUALITY_LEVEL_OPTION
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True, aliases=["fabric_template"]),
        name=dict(type="str", aliases=["node_setting"]),
        uuid=dict(type="str", aliases=["node_setting_uuid"]),
        description=dict(type="str"),
        synce=dict(
            type="dict",
            options=dict(
                state=dict(type="str", choices=["enabled", "disabled"]),
                admin_state=dict(type="str", choices=["enabled", "disabled"]),
                quality_level=dict(type="str", choices=list(SYNC_E_QUALITY_LEVEL_OPTION)),
            ),
            required_if=[["state", "enabled", ["admin_state", "quality_level"]]],
        ),
        ptp=dict(
            type="dict",
            options=dict(
                state=dict(type="str", choices=["enabled", "disabled"]),
                node_domain=dict(type="int"),
                priority_2=dict(type="int"),
            ),
            required_if=[["state", "enabled", ["node_domain", "priority_2"]]],
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    synce = module.params.get("synce")
    ptp = module.params.get("ptp")
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "fabric_policy", template)
    mso_template.validate_template("fabricPolicy")

    node_setting = mso_template.get_node_settings_object(uuid, name)

    if (uuid or name) and node_setting:
        mso.existing = mso.previous = copy.deepcopy(node_setting.details)  # Query a specific object
    elif node_setting:
        mso.existing = node_setting  # Query all objects

    if state != "query":
        node_setting_path = "/fabricPolicyTemplate/template/nodePolicyGroups/{0}".format(node_setting.index if node_setting else "-")

    ops = []
    if state == "present":
        if mso.existing:
            proposed_payload = copy.deepcopy(mso.existing)
            mso_values = dict()
            mso_values_remove = list()

            mso_values["name"] = name
            mso_values["description"] = description

            if synce is not None:
                if synce.get("state") == "disabled" and proposed_payload.get("synce"):
                    mso_values_remove.append("synce")
                else:
                    mso_values["synce"] = dict()
                    mso_values[("synce", "adminState")] = synce.get("admin_state")
                    mso_values[("synce", "qlOption")] = SYNC_E_QUALITY_LEVEL_OPTION.get(synce.get("quality_level"))

            if ptp is not None:
                if ptp.get("state") == "disabled" and proposed_payload.get("ptp"):
                    mso_values_remove.append("ptp")
                else:
                    mso_values["ptp"] = dict()

                    # Add the Priority 1 fixed value 128 to the PTP settings during initialization
                    mso_values[("ptp", "prio1")] = 128
                    mso_values[("ptp", "domain")] = ptp.get("node_domain")
                    mso_values[("ptp", "prio2")] = ptp.get("priority_2")

            append_update_ops_data(ops, proposed_payload, node_setting_path, mso_values, mso_values_remove)
            mso.sanitize(proposed_payload)
        else:
            mso_values = dict(name=name)

            if description:
                mso_values["description"] = description

            if synce is not None:
                mso_values["synce"] = dict(adminState=synce.get("admin_state"), qlOption=SYNC_E_QUALITY_LEVEL_OPTION.get(synce.get("quality_level")))

            if ptp is not None:
                mso_values["ptp"] = dict(
                    domain=ptp.get("node_domain"),
                    prio2=ptp.get("priority_2"),
                    prio1=128,
                )

            ops.append(dict(op="add", path=node_setting_path, value=copy.deepcopy(mso_values)))

            mso.sanitize(mso_values)
    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path=node_setting_path))

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        node_setting = mso_template.get_node_settings_object(uuid, name)
        if node_setting:
            mso.existing = node_setting.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019 Dell EMC Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: idrac_redfish_info
short_description: Gather PowerEdge server information through iDRAC using Redfish APIs
description:
  - Builds Redfish URIs locally and sends them to remote iDRAC controllers to get information back.
  - For use with Dell EMC iDRAC operations that require Redfish OEM extensions.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
  - community.general._redfish
attributes:
  check_mode:
    version_added: 3.3.0
    # This was backported to 2.5.4 and 1.3.11 as well, since this was a bugfix
options:
  category:
    required: true
    description:
      - Category to execute on iDRAC.
    type: str
  command:
    required: true
    description:
      - List of commands to execute on iDRAC.
      - V(GetManagerAttributes) returns the list of dicts containing iDRAC, LifecycleController and System attributes.
    type: list
    elements: str
  baseuri:
    required: true
    description:
      - Base URI of iDRAC.
    type: str
  username:
    description:
      - Username for authenticating to iDRAC.
    type: str
  password:
    description:
      - Password for authenticating to iDRAC.
    type: str
  auth_token:
    description:
      - Security token for authenticating to iDRAC.
    type: str
    version_added: 2.3.0
  timeout:
    description:
      - Timeout in seconds for HTTP requests to iDRAC.
    default: 10
    type: int
  validate_certs:
    version_added: 10.6.0
  ca_path:
    version_added: 10.6.0
  ciphers:
    version_added: 10.6.0

author: "Jose Delarosa (@jose-delarosa)"
"""

EXAMPLES = r"""
- name: Get Manager attributes with a default of 20 seconds
  community.general.idrac_redfish_info:
    category: Manager
    command: GetManagerAttributes
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
    timeout: 20
  register: result

# Examples to display the value of all or a single iDRAC attribute
- name: Store iDRAC attributes as a fact variable
  ansible.builtin.set_fact:
    idrac_attributes: "{{ result.redfish_facts.entries | selectattr('Id', 'defined') | selectattr('Id', 'equalto', 'iDRACAttributes')
      | list | first }}"

- name: Display all iDRAC attributes
  ansible.builtin.debug:
    var: idrac_attributes

- name: Display the value of 'Syslog.1.SysLogEnable' iDRAC attribute
  ansible.builtin.debug:
    var: idrac_attributes['Syslog.1.SysLogEnable']

# Examples to display the value of all or a single LifecycleController attribute
- name: Store LifecycleController attributes as a fact variable
  ansible.builtin.set_fact:
    lc_attributes: "{{ result.redfish_facts.entries | selectattr('Id', 'defined') | selectattr('Id', 'equalto', 'LCAttributes')
      | list | first }}"

- name: Display LifecycleController attributes
  ansible.builtin.debug:
    var: lc_attributes

- name: Display the value of 'CollectSystemInventoryOnRestart' attribute
  ansible.builtin.debug:
    var: lc_attributes['LCAttributes.1.CollectSystemInventoryOnRestart']

# Examples to display the value of all or a single System attribute
- name: Store System attributes as a fact variable
  ansible.builtin.set_fact:
    system_attributes: "{{ result.redfish_facts.entries | selectattr('Id', 'defined') | selectattr('Id', 'equalto', 'SystemAttributes')
      | list | first }}"

- name: Display System attributes
  ansible.builtin.debug:
    var: system_attributes

- name: Display the value of 'PSRedPolicy'
  ansible.builtin.debug:
    var: system_attributes['ServerPwr.1.PSRedPolicy']
"""

RETURN = r"""
msg:
  description: Different results depending on task.
  returned: always
  type: dict
  sample: List of Manager attributes
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._redfish_utils import (
    REDFISH_COMMON_ARGUMENT_SPEC,
    RedfishUtils,
)


class IdracRedfishUtils(RedfishUtils):
    def get_manager_attributes(self):
        result = {}
        response = {}
        manager_attributes = []
        properties = ["Attributes", "Id"]

        if len(self.manager_uris) == 1:
            response = self.get_request(f"{self.root_uri}{self.manager_uri}")
        elif len(self.manager_uris) > 1:
            response = self.get_request(f"{self.root_uri}/redfish/v1/Managers/iDRAC.Embedded.1", override_headers=None)

        if response["ret"] is False:
            return response
        data = response["data"]

        # Manager attributes are supported as part of iDRAC OEM extension
        # Attributes are supported only on iDRAC9
        try:
            for members in data["Links"]["Oem"]["Dell"]["DellAttributes"]:
                attributes_uri = members["@odata.id"]

                response = self.get_request(self.root_uri + attributes_uri)
                if response["ret"] is False:
                    return response
                data = response["data"]

                attributes = {}
                for prop in properties:
                    if prop in data:
                        attributes[prop] = data.get(prop)

                if attributes:
                    manager_attributes.append(attributes)

            result["ret"] = True

        except (AttributeError, KeyError) as e:
            result["ret"] = False
            result["msg"] = f"Failed to find attribute/key: {e}"

        result["entries"] = manager_attributes
        return result


CATEGORY_COMMANDS_ALL = {"Manager": ["GetManagerAttributes"]}


def main():
    result = {}
    argument_spec = dict(
        category=dict(required=True),
        command=dict(required=True, type="list", elements="str"),
        baseuri=dict(required=True),
        username=dict(),
        password=dict(no_log=True),
        auth_token=dict(no_log=True),
        timeout=dict(type="int", default=10),
    )
    argument_spec.update(REDFISH_COMMON_ARGUMENT_SPEC)
    module = AnsibleModule(
        argument_spec,
        required_together=[
            ("username", "password"),
        ],
        required_one_of=[
            ("username", "auth_token"),
        ],
        mutually_exclusive=[
            ("username", "auth_token"),
        ],
        supports_check_mode=True,
    )

    category = module.params["category"]
    command_list = module.params["command"]

    # admin credentials used for authentication
    creds = {"user": module.params["username"], "pswd": module.params["password"], "token": module.params["auth_token"]}

    # timeout
    timeout = module.params["timeout"]

    # Build root URI
    root_uri = f"https://{module.params['baseuri']}"
    rf_utils = IdracRedfishUtils(creds, root_uri, timeout, module)

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(msg=f"Invalid Category '{category}'. Valid Categories = {list(CATEGORY_COMMANDS_ALL.keys())}")

    # Check that all commands are valid
    for cmd in command_list:
        # Fail if even one command given is invalid
        if cmd not in CATEGORY_COMMANDS_ALL[category]:
            module.fail_json(msg=f"Invalid Command '{cmd}'. Valid Commands = {CATEGORY_COMMANDS_ALL[category]}")

    # Organize by Categories / Commands

    if category == "Manager":
        # execute only if we find a Manager resource
        result = rf_utils._find_managers_resource()
        if result["ret"] is False:
            module.fail_json(msg=result["msg"])

        for command in command_list:
            if command == "GetManagerAttributes":
                result = rf_utils.get_manager_attributes()

    # Return data back or fail with proper message
    if result["ret"] is True:
        del result["ret"]
        module.exit_json(redfish_facts=result)
    else:
        module.fail_json(msg=result["msg"])


if __name__ == "__main__":
    main()



VALID_IP_PROTOCOLS = ["eigrp", "egp", "icmp", "icmpv6", "igmp", "igp", "l2tp", "ospfigp", "pim", "tcp", "udp", "unspecified"]

FILTER_PORT_MAPPING = {"443": "https", "25": "smtp", "80": "http", "53": "dns", "110": "pop3", "554": "rtsp", "20": "ftpData", "ftp": "ftpData"}

VALID_ETHER_TYPES = ["arp", "fcoe", "ip", "ipv4", "ipv6", "mac_security", "mpls_ucast", "trill", "unspecified"]

VALID_QOS_CLASSES = ["unspecified", "level1", "level2", "level3", "level4", "level5", "level6"]

# mapping dicts are used to normalize the proposed data to what the APIC expects, which will keep diffs accurate
ARP_FLAG_MAPPING = dict(arp_reply="reply", arp_request="req", unspecified="unspecified")

# ICMPv4 Types Mapping
ICMP4_MAPPING = dict(
    dst_unreachable="dst-unreach", echo="echo", echo_reply="echo-rep", src_quench="src-quench", time_exceeded="time-exceeded", unspecified="unspecified"
)

# ICMPv6 Types Mapping
ICMP6_MAPPING = dict(
    dst_unreachable="dst-unreach",
    echo_request="echo-req",
    echo_reply="echo-rep",
    neighbor_advertisement="nbr-advert",
    neighbor_solicitation="nbr-solicit",
    redirect="redirect",
    time_exceeded="time-exceeded",
    unspecified="unspecified",
)

TCP_FLAGS = dict(acknowledgment="ack", established="est", finish="fin", reset="rst", synchronize="syn", unspecified="unspecified")

SUBNET_CONTROL_MAPPING = {"nd_ra_prefix": "nd", "no_default_gateway": "no-default-gateway", "querier_ip": "querier", "unspecified": ""}
SUBNET_CONTROL_MAPPING_BD_SUBNET = {"nd_ra": "nd", "no_gw": "no-default-gateway", "querier_ip": "querier", "unspecified": ""}

NODE_TYPE_MAPPING = {"tier_2": "tier-2-leaf", "remote": "remote-leaf-wan", "virtual": "virtual", "unspecified": "unspecified"}

SPAN_DIRECTION_MAP = {"incoming": "in", "outgoing": "out", "both": "both"}

MATCH_TYPE_MAPPING = {"all": "All", "at_least_one": "AtleastOne", "at_most_one": "AtmostOne", "none": "None"}

IPV4_REGEX = r"^((25[0-5]|(2[0-4]|1\d|[1-9]|)\d)\.?\b){4}$"

VM_PROVIDER_MAPPING = dict(
    cloudfoundry="CloudFoundry",
    kubernetes="Kubernetes",
    microsoft="Microsoft",
    openshift="OpenShift",
    openstack="OpenStack",
    redhat="Redhat",
    vmware="VMware",
    nutanix="Nutanix",
)

SYSLOG_FORMATS = {"enhanced_log": "rfc5424-ts", "nxos": "nxos", "aci": "aci", "rfc5424_ts": "rfc5424-ts"}

VM_SCOPE_MAPPING = dict(
    cloudfoundry="cloudfoundry",
    kubernetes="kubernetes",
    microsoft="MicrosoftSCVMM",
    openshift="openshift",
    openstack="openstack",
    redhat="rhev",
    vmware="vm",
    nutanix="nutanix",
)

MATCH_TYPE_GROUP_MAPPING = {"all": "ALL", "all_in_pod": "ALL_IN_POD", "range": "range"}

MATCH_FC_FILL_PATTERN_MAPPING = {"arbff": "ARBFF", "idle": "IDLE"}

MATCH_FIRMWARE_NODES_TYPE_MAPPING = {
    "c_apic_patch": "cApicPatch",
    "catalog": "catalog",
    "config": "config",
    "controller": "controller",
    "controller_patch": "controllerPatch",
    "plugin": "plugin",
    "plugin_package": "pluginPackage",
    "switch": "switch",
    "switch_patch": "switchPatch",
    "vpod": "vpod",
}

MATCH_TRIGGER_MAPPING = {
    "trigger": "trigger",
    "trigger_immediate": "trigger-immediate",
    "triggered": "triggered",
    "untriggered": "untriggered",
}

INTERFACE_POLICY_FC_SPEED_LIST = ["auto", "unknown", "2G", "4G", "8G", "16G", "32G"]

MATCH_RUN_MODE_MAPPING = dict(
    pause_always_between_sets="pauseAlwaysBetweenSets",
    pause_only_on_failures="pauseOnlyOnFailures",
    pause_never="pauseNever",
)

MATCH_NOTIFY_CONDITION_MAPPING = dict(
    notify_always_between_sets="notifyAlwaysBetweenSets",
    notify_never="notifyNever",
    notify_only_on_failures="notifyOnlyOnFailures",
)

MATCH_SMU_OPERATION_MAPPING = dict(smu_install="smuInstall", smu_uninstall="smuUninstall")

MATCH_SMU_OPERATION_FLAGS_MAPPING = dict(smu_reload_immediate="smuReloadImmediate", smu_reload_skip="smuReloadSkip")

MATCH_BEST_PATH_CONTROL_MAPPING = dict(enable="asPathMultipathRelax", disable="")

MATCH_GRACEFUL_RESTART_CONTROLS_MAPPING = dict(helper="helper", complete="")

EP_LOOP_PROTECTION_ACTION_MAPPING = {"bd": "bd-learn-disable", "port": "port-disable"}

FABRIC_POD_SELECTOR_TYPE_MAPPING = dict(all="ALL", range="range")

OPFLEX_TLS_MAPPING = {"tls_v1.0": "TLSv1", "tls_v1.1": "TLSv1.1", "tls_v1.2": "TLSv1.2"}

HTTP_TLS_MAPPING = {"tls_v1.0": "TLSv1", "tls_v1.1": "TLSv1.1", "tls_v1.2": "TLSv1.2", "tls_v1.3": "TLSv1.3"}

ACI_ACCESS_SWITCH_POLICY_GROUP_CLASS_MAPPING = dict(
    spine=dict(
        class_name="infraSpineAccNodePGrp",
        rn="infra/funcprof/spaccnodepgrp-{0}",
        copp_pre_filter_policy=dict(class_name="infraRsIaclSpineProfile", tn_name="tnIaclSpineProfileName"),
        bfd_ipv4_policy=dict(class_name="infraRsSpineBfdIpv4InstPol", tn_name="tnBfdIpv4InstPolName"),
        bfd_ipv6_policy=dict(class_name="infraRsSpineBfdIpv6InstPol", tn_name="tnBfdIpv6InstPolName"),
        copp_policy=dict(class_name="infraRsSpineCoppProfile", tn_name="tnCoppSpineProfileName"),
        cdp_policy=dict(class_name="infraRsSpinePGrpToCdpIfPol", tn_name="tnCdpIfPolName"),
        lldp_policy=dict(class_name="infraRsSpinePGrpToLldpIfPol", tn_name="tnLldpIfPolName"),
        usb_configuration_policy=dict(class_name="infraRsSpineTopoctrlUsbConfigProfilePol", tn_name="tnTopoctrlUsbConfigProfilePolName"),
    ),
    leaf=dict(
        class_name="infraAccNodePGrp",
        rn="infra/funcprof/accnodepgrp-{0}",
        copp_pre_filter_policy=dict(class_name="infraRsIaclLeafProfile", tn_name="tnIaclLeafProfileName"),
        bfd_ipv4_policy=dict(class_name="infraRsBfdIpv4InstPol", tn_name="tnBfdIpv4InstPolName"),
        bfd_ipv6_policy=dict(class_name="infraRsBfdIpv6InstPol", tn_name="tnBfdIpv6InstPolName"),
        copp_policy=dict(class_name="infraRsLeafCoppProfile", tn_name="tnCoppLeafProfileName"),
        cdp_policy=dict(class_name="infraRsLeafPGrpToCdpIfPol", tn_name="tnCdpIfPolName"),
        lldp_policy=dict(class_name="infraRsLeafPGrpToLldpIfPol", tn_name="tnLldpIfPolName"),
        usb_configuration_policy=dict(class_name="infraRsLeafTopoctrlUsbConfigProfilePol", tn_name="tnTopoctrlUsbConfigProfilePolName"),
    ),
)

PIM_SETTING_CONTROL_STATE_MAPPING = {"fast": "fast-conv", "strict": "strict-rfc-compliant"}

ACI_CLASS_MAPPING = dict(
    consumer={
        "class": "fvRsCons",
        "rn": "rscons-",
        "name": "tnVzBrCPName",
    },
    provider={
        "class": "fvRsProv",
        "rn": "rsprov-",
        "name": "tnVzBrCPName",
    },
    oob_provider={
        "class": "mgmtRsOoBProv",
        "rn": "rsooBProv-",
        "name": "tnVzOOBBrCPName",
    },
    taboo={
        "class": "fvRsProtBy",
        "rn": "rsprotBy-",
        "name": "tnVzTabooName",
    },
    interface={
        "class": "fvRsConsIf",
        "rn": "rsconsIf-",
        "name": "tnVzCPIfName",
    },
    intra_epg={
        "class": "fvRsIntraEpg",
        "rn": "rsintraEpg-",
        "name": "tnVzBrCPName",
    },
    intra_esg={
        "class": "fvRsIntraEpg",
        "rn": "rsintraEpg-",
        "name": "tnVzBrCPName",
    },
)

PROVIDER_MATCH_MAPPING = dict(
    all="All",
    at_least_one="AtleastOne",
    at_most_one="AtmostOne",
    none="None",
)

CONTRACT_LABEL_MAPPING = dict(
    consumer="vzConsLbl",
    provider="vzProvLbl",
)

SUBJ_LABEL_MAPPING = dict(
    consumer="vzConsSubjLbl",
    provider="vzProvSubjLbl",
)

SUBJ_LABEL_RN = dict(
    consumer="conssubjlbl-",
    provider="provsubjlbl-",
)

MATCH_ACTION_RULE_SET_METRIC_TYPE_MAPPING = {"ospf_type_1": "ospf-type1", "ospf_type_2": "ospf-type2", "": ""}

MATCH_EIGRP_INTERFACE_POLICY_DELAY_UNIT_MAPPING = dict(picoseconds="pico", tens_of_microseconds="tens-of-micro")

MATCH_EIGRP_INTERFACE_POLICY_CONTROL_STATE_MAPPING = dict(bfd="bfd", nexthop_self="nh-self", passive="passive", split_horizon="split-horizon")

MATCH_TARGET_COS_MAPPING = {
    "background": "0",
    "best_effort": "1",
    "excellent_effort": "2",
    "critical_applications": "3",
    "video": "4",
    "voice": "5",
    "internetwork_control": "6",
    "network_control": "7",
    "unspecified": "unspecified",
}

MATCH_PIM_INTERFACE_POLICY_CONTROL_STATE_MAPPING = dict(multicast_domain_boundary="border", strict_rfc_compliant="strict-rfc-compliant", passive="passive")

MATCH_PIM_INTERFACE_POLICY_AUTHENTICATION_TYPE_MAPPING = dict(none="none", md5_hmac="ah-md5")

MATCH_COLLECT_NETFLOW_RECORD_MAPPING = dict(
    bytes_counter="count-bytes",
    pkts_counter="count-pkts",
    pkt_disposition="pkt-disp",
    sampler_id="sampler-id",
    source_interface="src-intf",
    tcp_flags="tcp-flags",
    first_pkt_timestamp="ts-first",
    recent_pkt_timestamp="ts-recent",
)

MATCH_MATCH_NETFLOW_RECORD_MAPPING = dict(
    destination_ipv4_v6="dst-ip",
    destination_ipv4="dst-ipv4",
    destination_ipv6="dst-ipv6",
    destination_mac="dst-mac",
    destination_port="dst-port",
    ethertype="ethertype",
    ip_protocol="proto",
    source_ipv4_v6="src-ip",
    source_ipv4="src-ipv4",
    source_ipv6="src-ipv6",
    source_mac="src-mac",
    source_port="src-port",
    ip_tos="tos",
    unspecified="unspecified",
    vlan="vlan",
)

MATCH_SOURCE_IP_TYPE_NETFLOW_EXPORTER_MAPPING = dict(
    custom_source_ip="custom-src-ip",
    inband_management_ip="inband-mgmt-ip",
    out_of_band_management_ip="oob-mgmt-ip",
    ptep="ptep",
)

ECC_CURVE = {"P256": "prime256v1", "P384": "secp384r1", "P521": "secp521r1", "none": "none"}

THROTTLE_UNIT = dict(requests_per_second="r/s", requests_per_minute="r/m")

SSH_CIPHERS = dict(
    aes128_ctr="aes128-ctr",
    aes192_ctr="aes192-ctr",
    aes256_ctr="aes256-ctr",
    aes128_gcm="aes128-gcm@openssh.com",
    aes256_gcm="aes256-gcm@openssh.com",
    chacha20="chacha20-poly1305@openssh.com",
)

SSH_MACS = dict(
    sha1="hmac-sha1",
    sha2_256="hmac-sha2-256",
    sha2_512="hmac-sha2-512",
    sha2_256_etm="hmac-sha2-256-etm@openssh.com",
    sha2_512_etm="hmac-sha2-512-etm@openssh.com",
)

KEX_ALGORITHMS = dict(
    dh_sha1="diffie-hellman-group14-sha1",
    dh_sha256="diffie-hellman-group14-sha256",
    dh_sha512="diffie-hellman-group16-sha512",
    curve_sha256="curve25519-sha256",
    curve_sha256_libssh="curve25519-sha256@libssh.org",
    ecdh_256="ecdh-sha2-nistp256",
    ecdh_384="ecdh-sha2-nistp384",
    ecdh_521="ecdh-sha2-nistp521",
)

USEG_ATTRIBUTE_MAPPING = dict(
    ip=dict(attribute_type="ip", attribute_class="fvIpAttr", rn_format="ipattr-{0}"),
    mac=dict(attribute_type="mac", attribute_class="fvMacAttr", rn_format="macattr-{0}"),
    dns=dict(attribute_type="dns", attribute_class="fvDnsAttr", rn_format="dnsattr-{0}"),
    ad_group=dict(attribute_type="ad", attribute_class="fvIdGroupAttr", rn_format="idgattr-[{0}]"),
    vm_custom_attr=dict(attribute_type="custom-label", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_vmm_domain=dict(attribute_type="domain", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_operating_system=dict(attribute_type="guest-os", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_hypervisor_id=dict(attribute_type="hv", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_datacenter=dict(attribute_type="rootContName", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_id=dict(attribute_type="vm", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_name=dict(attribute_type="vm-name", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_folder=dict(attribute_type="vm-folder", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_folder_path=dict(attribute_type="vmfolder-path", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_vnic=dict(attribute_type="vnic", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
    vm_tag=dict(attribute_type="tag", attribute_class="fvVmAttr", rn_format="vmattr-{0}"),
)

OPERATOR_MAPPING = dict(equals="equals", contains="contains", starts_with="startsWith", ends_with="endsWith")

MATCH_STORM_CONTROL_POLICY_TYPE_MAPPING = dict(all_types="Invalid", unicast_broadcast_multicast="Valid")

POLICY_LABEL_COLORS = [
    "alice_blue",
    "antique_white",
    "aqua",
    "aquamarine",
    "azure",
    "beige",
    "bisque",
    "black",
    "blanched_almond",
    "blue",
    "blue_violet",
    "brown",
    "burlywood",
    "cadet_blue",
    "chartreuse",
    "chocolate",
    "coral",
    "cornflower_blue",
    "cornsilk",
    "crimson",
    "cyan",
    "dark_blue",
    "dark_cyan",
    "dark_goldenrod",
    "dark_gray",
    "dark_green",
    "dark_khaki",
    "dark_magenta",
    "dark_olive_green",
    "dark_orange",
    "dark_orchid",
    "dark_red",
    "dark_salmon",
    "dark_sea_green",
    "dark_slate_blue",
    "dark_slate_gray",
    "dark_turquoise",
    "dark_violet",
    "deep_pink",
    "deep_sky_blue",
    "dim_gray",
    "dodger_blue",
    "fire_brick",
    "floral_white",
    "forest_green",
    "fuchsia",
    "gainsboro",
    "ghost_white",
    "gold",
    "goldenrod",
    "gray",
    "green",
    "green_yellow",
    "honeydew",
    "hot_pink",
    "indian_red",
    "indigo",
    "ivory",
    "khaki",
    "lavender",
    "lavender_blush",
    "lawn_green",
    "lemon_chiffon",
    "light_blue",
    "light_coral",
    "light_cyan",
    "light_goldenrod_yellow",
    "light_gray",
    "light_green",
    "light_pink",
    "light_salmon",
    "light_sea_green",
    "light_sky_blue",
    "light_slate_gray",
    "light_steel_blue",
    "light_yellow",
    "lime",
    "lime_green",
    "linen",
    "magenta",
    "maroon",
    "medium_aquamarine",
    "medium_blue",
    "medium_orchid",
    "medium_purple",
    "medium_sea_green",
    "medium_slate_blue",
    "medium_spring_green",
    "medium_turquoise",
    "medium_violet_red",
    "midnight_blue",
    "mint_cream",
    "misty_rose",
    "moccasin",
    "navajo_white",
    "navy",
    "old_lace",
    "olive",
    "olive_drab",
    "orange",
    "orange_red",
    "orchid",
    "pale_goldenrod",
    "pale_green",
    "pale_turquoise",
    "pale_violet_red",
    "papaya_whip",
    "peachpuff",
    "peru",
    "pink",
    "plum",
    "powder_blue",
    "purple",
    "red",
    "rosy_brown",
    "royal_blue",
    "saddle_brown",
    "salmon",
    "sandy_brown",
    "sea_green",
    "seashell",
    "sienna",
    "silver",
    "sky_blue",
    "slate_blue",
    "slate_gray",
    "snow",
    "spring_green",
    "steel_blue",
    "tan",
    "teal",
    "thistle",
    "tomato",
    "turquoise",
    "violet",
    "wheat",
    "white",
    "white_smoke",
    "yellow",
    "yellow_green",
]

MATCH_ACCESS_POLICIES_SELECTOR_TYPE = dict(range="range", all="ALL")

MANAGEMENT_EPG_TYPE = dict(ooband="oob", inband="inb")

MANAGEMENT_EPG_CLASS_MAPPING = dict(in_band={"epg_class": "mgmtInB", "epg_rn": "inb-"}, out_of_band={"epg_class": "mgmtOoB", "epg_rn": "oob-"})

NODE_TYPE_MAPPING = {"tier_2": "tier-2-leaf", "remote": "remote-leaf-wan", "virtual": "virtual", "unspecified": "unspecified"}

SPAN_DIRECTION_MAP = {"incoming": "in", "outgoing": "out", "both": "both"}

HTTP_VERSIONS_MAPPING = {"1.0": "HTTP10", "1.1": "HTTP11"}

L4L7_FUNC_TYPES_MAPPING = {"go_to": "GoTo", "go_through": "GoThrough", "l1": "L1", "l2": "L2"}

L4L7_HASH_ALGORITHMS_MAPPING = {"source_ip": "sip", "destination_ip": "dip", "ip_and_protocol": "sip-dip-prototype"}

L4L7_FUNCTIONAL_TEMPLATE_TYPES_MAPPING = {
    "adc_one_arm": "ADC_ONE_ARM",
    "adc_two_arm": "ADC_TWO_ARM",
    "cloud_native_fw": "CLOUD_NATIVE_FW",
    "cloud_native_lb": "CLOUD_NATIVE_LB",
    "cloud_vendor_fw": "CLOUD_VENDOR_FW",
    "cloud_vendor_lb": "CLOUD_VENDOR_LB",
    "fw_routed": "FW_ROUTED",
    "fw_trans": "FW_TRANS",
    "other": "OTHER",
}

L4L7_UI_TEMPLATE_TYPE = {
    "ndo_implicit_template": "NDO_IMPLICIT_TEMPLATE",
    "one_node_adc_one_arm": "ONE_NODE_ADC_ONE_ARM",
    "one_node_adc_one_arm_l3ext": "ONE_NODE_ADC_ONE_ARM_L3EXT",
    "one_node_adc_two_arm": "ONE_NODE_ADC_TWO_ARM",
    "one_node_fw_routed": "ONE_NODE_FW_ROUTED",
    "one_node_fw_trans": "ONE_NODE_FW_TRANS",
    "two_node_fw_routed_adc_one_arm": "TWO_NODE_FW_ROUTED_ADC_ONE_ARM",
    "two_node_fw_routed_adc_one_arm_l3ext": "TWO_NODE_FW_ROUTED_ADC_ONE_ARM_L3EXT",
    "two_node_fw_routed_adc_two_arm": "TWO_NODE_FW_ROUTED_ADC_TWO_ARM",
    "two_node_fw_trans_adc_one_arm": "TWO_NODE_FW_TRANS_ADC_ONE_ARM",
    "two_node_fw_trans_adc_one_arm_l3ext": "TWO_NODE_FW_TRANS_ADC_ONE_ARM_L3EXT",
    "two_node_fw_trans_adc_two_arm": "TWO_NODE_FW_TRANS_ADC_TWO_ARM",
    "unspecified": "UNSPECIFIED",
}

COS_MAPPING = {"cos_0": "Cos0", "cos_1": "Cos1", "cos_2": "Cos2", "cos_3": "Cos3", "cos_4": "Cos4", "cos_5": "Cos5", "cos_6": "Cos6", "cos_7": "Cos7"}

RESERVED_ANSIBLE_INVENTORY_KEYS = {
    "serial": "_serial",
    "name": "_name",
}

MOCKED_CONSTRUCTED_INVENTORY_ARGUMENT_SPEC = dict(
    plugin=dict(type="str"),
    use_vars_plugins=dict(type="bool"),
    strict=dict(type="bool"),
    compose=dict(type="dict"),
    groups=dict(type="dict"),
    keyed_groups=dict(
        type="list",
        elements="dict",
        options=dict(
            parent_group=dict(type="str"),
            prefix=dict(type="str"),
            separator=dict(type="str"),
            key=dict(type="str"),
            default_value=dict(type="str"),
            trailing_separator=dict(type="bool"),
        ),
    ),
    use_extra_vars=dict(type="bool"),
    leading_separator=dict(type="bool"),
)

SWITCH_CONFIG_FORMAT_MAP = {
    "fabricNodeConfig": {
        "rn": "fabric/nodeconfnode-{0}",
        "type": "fabric",
        "spine": "uni/fabric/funcprof/spnodepgrp-{0}",
        "leaf": "uni/fabric/funcprof/lenodepgrp-{0}",
    },
    "infraNodeConfig": {
        "rn": "infra/nodeconfnode-{0}",
        "type": "access",
        "spine": "uni/infra/funcprof/spaccnodepgrp-{0}",
        "leaf": "uni/infra/funcprof/accnodepgrp-{0}",
    },
}

CONTRACT_CLASS_MAPPING = {"standard": {"class": "vzBrCP", "rn": "brc-{0}"}, "oob": {"class": "vzOOBBrCP", "rn": "oobbrc-{0}"}}
